/*
modules.cc
Copyright 2007 Matt Heinzen

This file is part of Color Machine 0.8

Color Machine is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Color Machine is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Color Machine; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "modules.h"
#include "colormachinesdl.h"

#include "utils.h"

#include <math.h>

void DrawModuleDiamonds()
{
	// draw module "Diamonds" in the display

	Uint8 ColorIndex;
	int nDiamond, Size, Flip;
	int x, y, i, j;

	nDiamond = Random(40) + 20;
	Size = Random(g_DisplayWidth / 16) + g_DisplayWidth / 16;
	ColorIndex = 1;

	for (i = 0; i < nDiamond; i++)
	{
		x = Random(g_DisplayWidth);
		y = Random(g_DisplayHeight);
		if (Random(2)) Flip = 1;
		else Flip = -1;

		if (Random(2))
		{
			for (j = -Size; j <= Size; j++, ColorIndex++)
			{
				if (ColorIndex == 0) ColorIndex++;

				DrawVLine(x + j * Flip, y - abs(j) + Size, y + abs(j) - Size,
					ColorIndex);
			}
		}
		else
		{
			for (j = -Size; j <= Size; j++, ColorIndex++)
			{
				if (ColorIndex == 0) ColorIndex++;

				DrawHLine(x - abs(j) + Size, x + abs(j) - Size, y + j * Flip,
					ColorIndex);
			}
		}
	}
}

void DrawModuleGrid()
{
	// draw module "Grid" in the display

	Uint8 ColorIndex;
	int i, j;

	ColorIndex = 1;

	for (i = 0; i < g_DisplayWidth; i += 2, ColorIndex++)
	{
		if (ColorIndex == 0) ColorIndex++;

		DrawVLine(i + 1, 0, g_DisplayHeight - 1, ColorIndex);
	}

	for (i = 0; i < g_DisplayHeight; i += 2, ColorIndex++)
	{
		if (ColorIndex == 0) ColorIndex++;

		DrawHLine(0, g_DisplayWidth - 1, i + 1, ColorIndex);
	}
}

void DrawModuleOrganic()
{
	// draw module "Organic" in the display

	int HalfWidth, HalfHeight, Range, HalfRange, MinX, MinY, MaxX, MaxY, Inc;
	int nNodes, ColorIndex;
	int x, y, oldx, oldy, i, j;

	HalfWidth = g_DisplayWidth / 2;
	HalfHeight = g_DisplayHeight / 2;
	Range = g_DisplayWidth / 15;
	Range += Range % 2;
	HalfRange = Range / 2;
	MinX = g_DisplayWidth / 8;
	MinY = g_DisplayHeight / 8;
	MaxX = g_DisplayWidth - MinX;
	MaxY = g_DisplayHeight - MinY;
	Inc = (g_DisplayWidth + g_DisplayHeight) / 128;

	nNodes = Random(1500) + 1000;
	ColorIndex = 1;

	oldx = Random(HalfWidth) + g_DisplayWidth / 4;
	oldy = Random(HalfHeight) + g_DisplayHeight / 4;

	for (i = 0; i < nNodes; i++, ColorIndex++)
	{
		if (ColorIndex == 0) ColorIndex = 1;

		x = oldx + Random(Range) - HalfRange;
		y = oldy + Random(Range) - HalfRange;

		if ((HalfWidth - x) * (HalfWidth - x)
			+ (HalfHeight - y) * (HalfHeight - y)
			> g_DisplayHeight * g_DisplayHeight / 4)
		{
			x -= (x - HalfWidth) / 4;
			y -= (y - HalfHeight) / 4;
		}

		DrawLine(oldx, oldy, x, y, ColorIndex);
		DrawLine(oldx - 1, oldy, x - 1, y, ColorIndex);
		DrawLine(oldx + 1, oldy, x + 1, y, ColorIndex);
		DrawLine(oldx, oldy - 1, x, y - 1, ColorIndex);
		DrawLine(oldx, oldy + 1, x, y + 1, ColorIndex);


		oldx = x;
		oldy = y;
	}
}

void DrawModuleSpikes()
{
	// draw module "Spikes" in the display

	bool bRound;
	Uint8 ColorIndex;
	int l, i, j, r, x, y, oldx, oldy;

	bRound = Random(2) == 1;

	l = g_DisplayWidth / 4;

	for (i = 3; i > -2; i--)
	{
		for (j = 2; j > -2; j--)
		{
		ColorIndex = Random(255) + 1;

		for (r = 0; r < l; r++, ColorIndex++)
		{
			if (ColorIndex == 0) ColorIndex++;

			x = i * l + r * 2;
			y = j * l + r * 2;
			oldx = x + l - r;
			oldy = y + l - r;

			/*if (x < g_DisplayWidth && y < g_DisplayHeight && oldx > 0
				&& oldy > 0)*/
			if (bRound) DrawDisc(x, y, l - r, ColorIndex);
			else DrawBox(x, y, oldx, oldy, ColorIndex);
      }
    }
  }
}

void DrawModuleWorms()
{
	// draw module "Worms" in the display

	Uint8 ColorIndex;
	int Size;
	int r, l, i, j, x, y, dx, dy;

	r = Random(g_DisplayWidth / 16) + g_DisplayWidth / 32;

	Size = Random(g_DisplayWidth / 128) + 10;

	for (i = 0; i < r; i++)
	{
		l = Random(500) + 500;
		ColorIndex = Random(255) + 1;
		x = Random(g_DisplayWidth);
		y = Random(g_DisplayHeight);

		for (j = 0; j < l; j++, ColorIndex++)
		{
			if (ColorIndex == 0) ColorIndex++;

			dx = Random(5) - 2;
			dy = Random(5) - 2;

			x += dx;
			y += dy;

			if (x < 0) x += 1;
			if (y < 0) y += 1;
			if (x > g_DisplayWidth - 10) x -= 1;
			if (y > g_DisplayHeight - 10) y -= 1;

			DrawDisc(x, y, (int)(Size * sin(M_PI_2 * j / l + M_PI_4)),
				ColorIndex);
		}
	}
}

void DrawModulePipeline()
{
	// draw module "Pipeline" in the display

	int PipeSize;
	bool bRound;
	Uint8 ColorIndex;
	int r, i, j, l, x, y, dx, dy;

	PipeSize = Random(g_DisplayWidth / 32) + g_DisplayWidth / 32;
	bRound = Random(2) == 1;

	r = Random(7) + 8;
	//r = 1;

	for (i = 0; i < r; i++)
	{
		ColorIndex = Random(255) + 1;
		//x = Random(285) + 10;
		//y = Random(165) + 10;
		x = Random(g_DisplayWidth);
		y = Random(g_DisplayHeight);

		while (x > -PipeSize && y > -PipeSize && x < g_DisplayWidth
			&& y < g_DisplayHeight)
		{
			l = Random(30) + 10;
			do dx = Random(5) - 2; while (dx == 0);
			do dy = Random(5) - 2; while (dy == 0);

			for (j = 0; j < l && x > -PipeSize && y > -PipeSize
				&& x < g_DisplayWidth && y < g_DisplayHeight;
				j++, ColorIndex++)
			{
				if (ColorIndex == 0) ColorIndex++;

				x += dx;
				y += dy;

				/*if (x > -PipeSize && y > -PipeSize && x < g_DisplayWidth
					&& y < g_DisplayHeight)*/
				if (bRound) DrawDisc(x, y, PipeSize / 2, ColorIndex);
				else DrawBox(x - PipeSize / 2, y - PipeSize / 2,
					x + PipeSize / 2, y + PipeSize / 2, ColorIndex);
			}
		}
	}
}

void DrawModuleFractured()
{
	// draw module "Fractured" in the display

	Uint8 ColorIndex;
	int Dim, Step;
	bool bRound;
	int l, r, i, j, x, y, dx, dy;

	Dim = g_DisplayWidth / 6;
	Step = g_DisplayWidth / 200;

	bRound = Random(2) == 1;

	l = Random(g_DisplayWidth / 50) + g_DisplayWidth / 100;
	r = Random(g_DisplayWidth / 15) + g_DisplayWidth / 300;

	ColorIndex = Random(255) + 1;

	for (i = -Dim; i < Dim; i++, ColorIndex++)
	{
		if (ColorIndex == 0) ColorIndex++;

		for (j = -Dim; j < Dim; j++)
		{
			dx = Random(r * 2 + 1) - r;
			dy = Random(r * 2 + 1) - r;
			x = i * Step + j * Step + dx + g_DisplayWidth / 2;
			y = i * Step - j * Step + dy + g_DisplayHeight / 2;

			if (x > -l && y > -l && x < g_DisplayWidth && y < g_DisplayHeight)
			{
				if (bRound) DrawBox(x, y, x + l, y + l, ColorIndex);
				else DrawDisc(x, y, l, ColorIndex);
			}
		}
	}
}

void DrawModuleCheckerboard()
{
	// draw module "Checkerboard" in the display

	Uint8 ColorIndex;
	int Size, nStepX, nStepY, Flip;
	int i, j, k;

	Size = g_DisplayWidth / 16;

	nStepX = g_DisplayWidth / Size + 1;
	nStepY = g_DisplayHeight / Size + 1;

	for (i = 0; i < nStepY; i++)
	{
		for (j = 0; j < nStepX; j++)
		{
			ColorIndex = Random(255) + 1;
			if (Random(2)) Flip = 1;
			else Flip = -1;

			for (k = 0; k < Size; k++, ColorIndex += Flip)
			{
				if (ColorIndex == 0) ColorIndex += Flip;

				if ((i + j) % 2) DrawVLine(j * Size + k, i * Size,
					(i + 1) * Size - 1, ColorIndex);
				else DrawHLine(j * Size, (j + 1) * Size - 1, i * Size + k,
					ColorIndex);
			}
		}
	}
}

void DrawModuleMicrochip()
{
	// draw module "Microchip" in the display

	Uint8 ColorIndex;
	int r, x, y;

	r = Random(13) + 1;

	for (y = 0; y < g_DisplayHeight; y++)
	{
		ColorIndex = Random(255) + 1;
		for (x = 0; x < g_DisplayWidth; x++)
		{
			ColorIndex += (Random(2 * r + 1) - r);
			if (ColorIndex == 0) ColorIndex++;
			PlotPixel(x, y, ColorIndex);
		}
	}

	for (x = 0; x < g_DisplayWidth; x += 2)
	{
		ColorIndex = Random(255) + 1;
		for (y = 0; y < g_DisplayHeight; y++)
		{
			ColorIndex += (Random(2 * r + 1) - r);
			if (ColorIndex == 0) ColorIndex++;
			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleOpposition()
{
	// draw module "Opposition" in the display

	int ColorIndex;
	int x, y, x2, y2, i, j, dx, dy, dx2, dy2;
	double l, d, r;

	do
	{
		x = Random(g_DisplayWidth);
		y = Random(g_DisplayHeight);
		x2 = Random(g_DisplayWidth);
		y2 = Random(g_DisplayHeight);
		l = sqrt(abs(x - x2) * abs(x - x2) + abs(y - y2) * abs(y - y2));
	} while (l < 50 || l > 200);

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for (j = 0; j < g_DisplayHeight; j++)
		{
			if (i == x && j == y) ColorIndex = 255;
			else if (i == x2 && j == y2) ColorIndex = 1;
			else
			{
				dx = abs(i - x);
				dy = abs(j - y);
				dx2 = abs(i - x2);
				dy2 = abs(j - y2);
				d = sqrt(dx * dx + dy * dy);
				r = sqrt(dx2 * dx2 + dy2 * dy2);
				ColorIndex = (Uint8)(128.0 + 127.0 * (d - r) / l);
			}

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleInfinity()
{
	// draw module "Infinity" in the display

	// REWRITE

	// NEED TO MODIFY PARAMETERS
	// TODO: switch c and ColorIndex???

	Uint8 ColorIndex;
	int Dist;
	int r, i, j, c, dy, dx, x, y;

	ColorIndex = Random(255) + 1;

	r = Random(MAX_3D - 900) + 900;
	for (Dist = 30000; Dist > 100; Dist -= 30)
	{
		for (i = 50; i <= MAX_3D; i += 150)
		{
			for (j = 50; j <= MAX_3D; j += 150)
			{
				c = ColorIndex + Dist / 50;
				while (c < 1) c += 255;
				while (c > 255) c -= 255;

				dy = 100 * i / Dist;
				dx = 100 * j / Dist;

				if (abs(dy < g_DisplayHeight / 2))
				{
					if (i > r)
					{
						y = g_DisplayHeight / 2 + dy;
						DrawHLine(0, g_DisplayWidth - 1, y, c);

						y = g_DisplayHeight / 2 - dy;
						DrawHLine(0, g_DisplayWidth - 1, y, c);
					}
					else if (j == MAX_3D)
					{
						if (abs(dx < g_DisplayWidth / 2))
						{
							y = g_DisplayHeight / 2 - dy;
							x = g_DisplayWidth / 2 + dx;
							DrawHLine(x, g_DisplayWidth - 1, y, c);

							x = g_DisplayWidth / 2 - dx;
							DrawHLine(0, x, y, c);

							y = g_DisplayHeight / 2 + dy;
							x = g_DisplayWidth / 2 + dx;
							DrawHLine(x, g_DisplayWidth - 1, y, c);

							x = g_DisplayWidth / 2 - dx;
							DrawHLine(0, x, y, c);
						}
					}
				}
				//c = 256 - c;

				if (abs(dx < g_DisplayWidth / 2))
				{
					if (j > r) {
						x = g_DisplayWidth / 2 + dx;
						DrawVLine(x, 0, g_DisplayHeight - 1, c);

						x = g_DisplayWidth / 2 - dx;
						DrawVLine(x, 0, g_DisplayHeight - 1, c);
					}
					else if (i == MAX_3D)
					{
						if (abs(dy < g_DisplayHeight / 2))
						{
						x = g_DisplayWidth / 2 - dx;
						y = g_DisplayHeight / 2 + dy;
						DrawVLine(x, y, g_DisplayHeight - 1, c);

						y = g_DisplayHeight / 2 - dy;
						DrawVLine(x, 0, y, c);

						x = g_DisplayWidth / 2 + dx;
						y = g_DisplayHeight / 2 + dy;
						DrawVLine(x, y, g_DisplayHeight - 1, c);

						y = g_DisplayHeight / 2 - dy;
						DrawVLine(x, 0, y, c);
						}
					}
				}/**/
			}

			// PREVIOUSLY REMOVED
			//for (l = 0; l < 5; l++)
			//{
			//  x = random(160) + random(160);
			//  y = random(100) + random(100);
			//  plot_pixel(x, y, 0);
			//}
			//for (l = 0; l < 10; l++)
			//{
			//  x = random(160) + random(160);
			//  if (x < 160) x = 160 - x;
			//  else x = 160 + 320 - x;
			//  y = random(100) + random(100);
			//  if (y < 100) y = 100 - y;
			//  else y = 100 + 200 - y;
			//  plot_pixel(x, y, c);
			//}
		}
	}
}

void DrawModuleContour()
{
	// draw module "Contour" in the display

	Uint8 ColorIndex;

	long c,  d,  i,  j,  l,  n,  r;
	long x,  y,  x2,  y2,  oldx,  oldy,  dx,  dy,  dx2,  dy2,  cx,  cy;
	long xh[5],  yh[5],  xl[5],  yl[5],  dxh[5],  dyh[5],  dxl[5],  dyl[5];
	double dha,  dla,  dh[5],  dl[5];
	int num_high,  num_low,  sign;

	do
	{
		num_high = Random(5) + 1;
		num_low = Random(5) + 1;
	} while (num_high < 2 && num_low < 2);

	r = Random(3);
	/**r = 2;/**/

	for (i = 0; i < num_high; i++)
	{
		xh[i] = Random(g_DisplayWidth);
		yh[i] = Random(g_DisplayHeight);
	}

	for (i = 0; i < num_low; i++)
	{
		xl[i] = Random(g_DisplayWidth);
		yl[i] = Random(g_DisplayHeight);
	}

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for (j = 0; j < g_DisplayHeight; j++)
		{
			for (l = 0; l < num_high; l++)
			{
				dxh[l] = abs(i - xh[l]);
				dyh[l] = abs(j - yh[l]);
			}

			for (l = 0; l < num_low; l++)
			{
				dxl[l] = abs(i - xl[l]);
				dyl[l] = abs(j - yl[l]);
			}

			for (l = 0; l < num_high; l++)
				dh[l] = sqrt(dxh[l] * dxh[l] + dyh[l] * dyh[l]);
			for (l = 0; l < num_low; l++)
				dl[l] = sqrt(dxl[l] * dxl[l] + dyl[l] * dyl[l]);

			dha = dh[0];
			if (dh[1] < dha && num_high > 1) dha = dh[1];
			if (dh[2] < dha && num_high > 2) dha = dh[2];
			if (dh[3] < dha && num_high > 3) dha = dh[3];
			if (dh[4] < dha && num_high > 4) dha = dh[4];

			dla = dl[0];
			if (dl[1] < dla && num_low > 1) dla = dl[1];
			if (dl[2] < dla && num_low > 2) dla = dl[2];
			if (dl[3] < dla && num_low > 3) dla = dl[3];
			if (dl[4] < dla && num_low > 4) dla = dl[4];

			if (dha < 1.0) dha = 1.0;
			if (dla < 1.0) dla = 1.0;

			// ColorIndex = 256 * dla / dha;  // option 1
			switch (r)
			{
			case 0:
				ColorIndex = Int2Color(int(128.0 + 13.0 * (dla - dha) / 10.0));
				break;
			case 1:
				ColorIndex = Int2Color(int(138.0 + 13.0 * (sqrt(dla) - sqrt(dha))));
				break;
			case 2:
				ColorIndex = Int2Color(int(138.0 + 100.0 * (sqrt(sqrt(dla))
					- sqrt(sqrt(dha)))));
				break;
			}

			/*if (ColorIndex == 0) ColorIndex++;*/

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleVortex()
{
	// draw module "Vortex" in the display

	// TODO: NEED TO TWEAK

	int ColorIndex;
	double theta, colorstep;
	int d, i, r, l, x, y;

	ColorIndex = Random(255) + 1;
	theta = 0;
	d = Random(3) + 1;
	colorstep = (double)(Random(128) + 64);

	for (i = 0; i < g_DisplayWidth * 2; i++)
	{
		theta += 0.02;
		//r = i / 3;
		/**/r = i * i / (g_DisplayWidth * g_DisplayWidth / 200);/**/
		l = int(r / 7);
		/**/x = int(g_DisplayWidth / 2 + 1.33 * r * cos(theta) - l / 2);
		y = int(g_DisplayHeight / 2 - r * sin(theta) - l / 2);/**/

		/**x = r * cos(theta) + g_DisplayWidth / 2;
		y = r * sin(theta) + g_DisplayHeight / 2;/**/

		ColorIndex = (Uint8)(theta * colorstep) % 255 + 1;

		DrawDisc(x, y, l, ColorIndex);
	}
}

void DrawModuleThorns()
{
	// draw module "Thorns" in the display

	Uint8 ColorIndex;
	int RangeX, RangeY, PadX, PadY, MinThickness, RangeThickness;
	int n, x, y, x2, y2, d, l, r, i, j;

	PadX = g_DisplayWidth / 20;
	PadY = g_DisplayHeight / 20;
	RangeX = g_DisplayWidth - PadX * 2;
	RangeY = g_DisplayHeight - PadY * 2;
	MinThickness = g_DisplayWidth / 15;
	RangeThickness = g_DisplayWidth / 10;

	n = Random(30) + 15;

	for (i = 0; i < n; i++)
	{
		ColorIndex = Random(255) + 1;
		x = Random(RangeX) + PadX;
		y = Random(RangeY) + PadY;
		d = Random(2);
		l = Random(RangeThickness) + MinThickness;
		r = Random(4);

		if (r <= 1)
		{
			if (r == 0) x2 = 0; else x2 = g_DisplayWidth - 1;
			y2 = Random(RangeY) + PadY;
			if (d > 0) {
				for (j = y2; j < y2 + l; j++, ColorIndex++)
				{
					if (ColorIndex == 0) ColorIndex = 1;
					if (j < g_DisplayHeight) DrawLine(x, y, x2, j, ColorIndex);
				}
			}
			else
			{
				for (j = y2; j > y2 - l; j--, ColorIndex++) {
					if (ColorIndex == 0) ColorIndex = 1;
					if (j >= 0) DrawLine(x, y, x2, j, ColorIndex);
				}
			}
		}
		else
		{
			if (r == 2) y2 = 0; else y2 = g_DisplayHeight - 1;
			x2 = Random(RangeX) + PadX;
			if (d > 0) {
				for (j = x2; j < x2 + l; j++, ColorIndex++)
				{
					if (ColorIndex == 0) ColorIndex = 1;
					if (j < g_DisplayWidth)
					DrawLine(x, y, j, y2, ColorIndex);
				}
			}
			else
			{
				for (j = x2; j > x2 - l; j--, ColorIndex++)
				{
					if (ColorIndex == 0) ColorIndex = 1;
					if (j > -1)
					DrawLine(x, y, j, y2, ColorIndex);
				}
			}
		}
	}
}

void DrawModuleWaves()
{
	// draw module "Waves" in the display

	Uint8 ColorIndex;
	double r, f;
	int x, y;

	r = Random(g_DisplayWidth / 50) + g_DisplayWidth / 200;
	f = (Random(16) + 4) / 8.0;

	for (x = 0; x < g_DisplayWidth; x++)
	{
		for (y = 0; y < g_DisplayHeight; y++)
		{
			ColorIndex = Int2Color((int)(64.0 * sin(x / r * f)
				+ 64.0 * sin(y / r)) + 128);
			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleNucleus()
{
	// draw module "Nucleus" in the display

	Uint8 ColorIndex;
	int x, y, x2, y2, r, n;
	double f, d;

	x2 = Random(g_DisplayWidth - g_DisplayWidth / 5) + g_DisplayWidth / 5;
	y2 = Random(g_DisplayHeight / 5) + g_DisplayHeight / 5;
	r = Random(20) + 5;
	f = float(Random(150) + 50) / 100 / 1.2;
	n = Random(5) + Random(3) + Random(2) + 2;

	for (x = 0; x < g_DisplayWidth; x++)
	{
		for (y = 0; y < g_DisplayHeight; y++)
		{
			d = hypot(abs(x - x2), abs(y - y2));

			ColorIndex = (Uint8)(64 * sin((double)(x) / r * f) +
				64 * sin((double)(y) / r) + d * n);
			if (ColorIndex == 0) ColorIndex++;

			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleJelly()
{
	// draw module "Jelly" in the display

	Uint8 ColorIndex;
	int r, x, y;
	double f;

	r = Random(20) + 5;
	f = double(Random(150) + 50) / 100 / 1.2;
	for (x = 0; x < g_DisplayWidth; x++)
	{
		for (y = 0; y < g_DisplayHeight; y++)
		{
			ColorIndex = (Uint8)(32.0 * sin((double)(x) / r * f) +
			32.0 * sin((double)(x * 3.0 / 2.0) / r * f) +
			32.0 * sin((double)(y) / r) +
			32.0 * sin((double)(y * 3.0 / 2.0) / r) + 128.0);
			if (ColorIndex == 0) ColorIndex++;

			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleRays()
{
	// draw module "Rays" in the display

	Uint8 ColorIndex;
	int x, y, x2, y2;
	double Period, dx, dy, theta;

	x2 = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	y2 = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;

	Period = M_PI * 2.0 / (double)(Random(8) + 3);

	for (x = 0; x < g_DisplayWidth; x++)
	{
		for (y = 0; y < g_DisplayHeight; y++)
		{
			dx = (double)(x - x2);
			dy = (double)(y2 - y);

			theta = atan2(dy, dx);
			ColorIndex = Int2Color(int(theta / Period * 255.0) + 1);
			//if (ColorIndex == 0) ColorIndex = 255;

			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleSpiral()
{
	// draw module "Spiral" in the display

	int ColorIndex;
	int x, y, x2, y2, dx, dy;
	double Flip, f, d, theta, n;

	x2 = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	y2 = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;

	if (Random(2) == 0) Flip = -1.0;
	else Flip = 1.0;
	f = M_PI * 2.0 / (double)(Random(5) + 2);
	n = (Random(20) + 1) / 5.0;

	for (x = 0; x < g_DisplayWidth; x++)
	{
		for (y = 0; y < g_DisplayHeight; y++)
		{
			dx = x - x2;
			dy = y2 - y;
			d = hypot(dx, dy);
			theta = Flip * atan2((double)dy, (double)dx);

			ColorIndex = Int2Color((int)(theta * 255 / f + d * n));
			PlotPixel(x, y, ColorIndex);
		}
	}
}

void DrawModuleOsmosis()
{
	// draw module "Osmosis" in the display

	// ORIGINAL USING INTEGER MATH
	/**
	int ColorIndex;
	int r, x, x2, y, y2, i, j, d, l;

	r = Random(8000 / g_DisplayWidth) + 2000 / g_DisplayWidth;
	x = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	x2 = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	y = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;
	y2 = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for (j = 0; j < g_DisplayHeight; j++)
		{
			d = (int)(hypot(i - x, j - y));
			l = (int)(hypot(i - x2, j - y2));
			ColorIndex = Int2Color(d * l * r);

			PlotPixel(i, j, ColorIndex);
		}
	}
	/**/

	// EXPERIMENTAL CONVERSION TO FP MATH
	// gives very different results, similar to ???
	// NOTE: d and l being ints before multiply give characteristic effect
	/**/
	int ColorIndex;
	int x, x2, y, y2, i, j;
	double r, d, l;

	r = 0.1 * (Random(20) + 1);
	x = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	x2 = Random(g_DisplayWidth / 2) + g_DisplayWidth / 4;
	y = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;
	y2 = Random(g_DisplayHeight / 2) + g_DisplayHeight / 4;

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for (j = 0; j < g_DisplayHeight; j++)
		{
			d = hypot(i - x, j - y);
			l = hypot(i - x2, j - y2);

			ColorIndex = Int2Color((int)(d * l * r)); // new behavior
			//ColorIndex = Int2Color((int)d * (int)l * r ); // old behavior
			//ColorIndex = Int2Color((int)((d + l) * r)); // ellipse

			PlotPixel(i, j, ColorIndex);
		}
	}
	/**/
}

void DrawModuleMultipolarity()
{
	// draw module "Multipolarity" in the display

	Uint8 ColorIndex;
	int NoCos;
	double r, x1, y1, x2, y2, x3, y3, cx, cy, d1, d2, d3;
	int i, j;

	r = Random(g_DisplayWidth / 6) + g_DisplayWidth / 30.0;
	NoCos = Random(3);

	x1 = Random(g_DisplayWidth);
	y1 = Random(g_DisplayHeight);
	x2 = Random(g_DisplayWidth);
	y2 = Random(g_DisplayHeight);
	x3 = Random(g_DisplayWidth);
	y3 = Random(g_DisplayHeight);

	if (Random(2) == 0) {
		cx = (x1 + x2 + x3) / 3;
		cy = (y1 + y2 + y3) / 3;
	}
	else {
		cx = Random(g_DisplayWidth);
		cy = Random(g_DisplayHeight);
	}

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for (j = 0; j < g_DisplayHeight; j++)
		{
			d1 = hypot(i - x1, j - y1);
			d2 = hypot(i - x2, j - y2);
			d3 = hypot(i - x3, j - y3);

			if (NoCos == 0) ColorIndex = Int2Color((int)((d1 + d2 + d3)
				* sin(hypot(i - cx, j - cy) / r)));
			else ColorIndex = Int2Color((int)((d1 + d2 + d3)
				* cos(sin(hypot(i - cx, j - cy) / r))));

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleRipples()
{
	// draw module "Ripples" in the display

	Uint8 ColorIndex;
	int x, y, x2, y2, i, j;
	float f, fx, fy;

	x = Random(g_DisplayWidth);
	y = Random(g_DisplayHeight);
	x2 = Random(g_DisplayWidth);
	y2 = Random(g_DisplayHeight);

	f = float(Random(g_DisplayWidth / 48)) + 0.750;

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for(j = 0; j < g_DisplayHeight; j++)
		{
			fx = (float(i - x) * float(i - x) +
				float(i - x2) * float(i - x2)) / f;
			fy = (float(j - y) * float(j - y) +
				float(j - y2) * float(j - y2)) / f;

			ColorIndex = Int2Color(int(fx + fy));

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleBeyond()
{
  // draw module "Beyond" in the display

	Uint8 ColorIndex;
	int x, y, x2, y2, i, j;
	float f, fx, fy;

	x = Random(g_DisplayWidth);
	y = Random(g_DisplayHeight);
	x2 = Random(g_DisplayWidth);
	y2 = Random(g_DisplayHeight);
	f = float(Random(g_DisplayWidth / 6)) + 8.0;

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for(j = 0; j < g_DisplayHeight; j++)
		{
			fx = (float(i - x) * float(i - x) +
				float(i - x2) * float(i - x2)) / f;
			fy = (float(j - y) * float(j - y) +
				float(j - y2) * float(j - y2)) / f;

			ColorIndex = Int2Color(int((fx - fy) * 10.0));

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleContinuum()
{
  // draw module "Continuum" in the display

	Uint8 ColorIndex;
	int x, y, x2, y2, i, j;
	float f, fx, fy;

	x = Random(g_DisplayWidth * 2) - g_DisplayWidth / 2;
	y = Random(g_DisplayHeight * 2) - g_DisplayHeight / 2;
	x2 = Random(g_DisplayWidth);
	y2 = Random(g_DisplayHeight);
	f = float(Random(g_DisplayWidth * 8)) + g_DisplayWidth / 4.0;

	for (i = 0; i < g_DisplayWidth; i++)
	{
		for(j = 0; j < g_DisplayHeight; j++)
		{
			fx = (float(i - x) * float(i - x) * float(i - x) +
				float(i - x2) * float(i - x2) * float(i - x2)) / f;
			fy = (float(j - y) * float(j - y)  * float(i - y) +
				float(j - y2) * float(j - y2)  * float(i - y2)) / f;

			ColorIndex = Int2Color(int(fx + fy));

			PlotPixel(i, j, ColorIndex);
		}
	}
}

void DrawModuleSpheres()
{
  // draw module "Spheres" in the display

	Uint8 ColorIndex;
	int CMax, CRange, CMin, CAvg;
	int n, x, y, r, x2, y2;
	double d, edgedist, focaldist;
	double theta;
	int i, j, k;

	n = Random(15) + 10;
	for (k = 0; k < n; k++)
	{
		CMin = Random(255);
		CRange = Random(5120) + 128;
		if (Random(2) == 0) CRange = -CRange;
		CMax = CMin + CRange;

		CMin = 1;
		CMax = 255;
		CRange = 254;

		CAvg = (CMax + CMin) / 2;

		x = Random(g_DisplayWidth);
		y = Random(g_DisplayHeight);
		r = Random(g_DisplayWidth / 6) + g_DisplayWidth / 50;

		theta = Random(360) * M_PI / 180.0;
		focaldist = Random(r) * 0.8;
		x2 = (int)(x + focaldist * cos(theta));
		y2 = (int)(y + focaldist * sin(theta));

		for (i = x - r; i <= x + r; i++)
		{
			for (j = y - r; j <= y + r; j++)
			{
				d = hypot(x - i, y - j);

				if (d <= r) {
					edgedist = 1.75 * r - d;
					focaldist = hypot(x2 - i, y2 - j);
					ColorIndex = Int2Color((int)(CMin + CRange
						* (focaldist - edgedist) / (focaldist + edgedist)));

					PlotPixel(i, j, ColorIndex);
				}
			}
		}
	}
}

void DrawModuleGodsOrb()
{
	// draw module "God's Orb" in the display

	Uint8 ColorIndex;
	int Mode;
	int sign;
	double x, y;
	double x1, x2, y1, y2;
	double dx, dy, d;
	double focaldist, edgedist, theta;
	double Radius;
	double Off, Off2;
	int i, j;

	x = g_DisplayWidth / 2;
	y = g_DisplayHeight / 2;
	Radius = (float)(g_DisplayHeight / 2) * 0.85f;
	Off = g_DisplayWidth / 20;
	Off2 = g_DisplayWidth / 10;
	Mode = Random(4) + 1;
	/**Mode = 4;/**/

	// display the background
	for (x2 = -g_DisplayWidth / 2; x2 < g_DisplayWidth / 2; x2 += 3)
	{
		for (y2 = -g_DisplayHeight / 2; y2 < g_DisplayHeight / 2; y2 += 3)
		{
			i = (int)(x2) + g_DisplayWidth / 2;
			j = (int)(y2) + g_DisplayHeight / 2;

			switch (Mode)
			{
			case 1:
				ColorIndex = Int2Color(i);
				if (j % 2 == 1) ColorIndex = Int2Color(j);
				break;
			case 2:
				ColorIndex = Int2Color((int)((sin(x2 / 8.0) + sin(y2 / 8.0))
					* 64.0));
				break;
			case 3:
				if (i % 2 == 1)
				{
					if (j % 2 == 1) ColorIndex = Int2Color((int)(3.0 * sqrt(
						(x2 - Off2) * (x2 - Off2) + (y2 - Off2)
						* (y2 - Off2))));
					else ColorIndex = Int2Color((int)(3.0 * sqrt(
						(x2 - Off2) * (x2 - Off2) + (y2 + Off2)
						* (y2 + Off2))));
				}
				else
				{
					if (j % 2 == 1) ColorIndex = Int2Color((int)(3.0 * sqrt(
						(x2 + Off2) * (x2 + Off2) + (y2 - Off2)
						* (y2 - Off2))));
					else ColorIndex = Int2Color((int)(3.0 * sqrt(
						(x2 + Off2) * (x2 + Off2) + (y2 + Off2)
						* (y2 + Off2))));
				}
				break;
			case 4:
				ColorIndex = Int2Color((int)(atan((double)y2
					/ (double)x2) * 255.0 / M_PI_2
					+ sqrt((double)(x2 * x2 + y2 * y2)) * 6.0));
				break;
			}

			PlotPixel(i, j, ColorIndex);
		}
	}

	// display the sphere
	for (i = (int)(x - Radius); i < x + Radius; i++)
	{
		for (j = (int)(y - Radius); j < y + Radius; j++)
		{
			focaldist = hypot((x - i), (y - j));
			if (focaldist <= Radius)
			{
				dx = i - x;
				dy = j - y;

				if (dx > 0.0) theta = atan(dy / dx);
				else if (dx < 0.0) theta = M_PI - atan(dy / -dx);
				else
				{
					if (dy > 0.0) theta = M_PI_2;
					else theta = 3 * M_PI_2;
				}

				edgedist = Radius - focaldist;

				// coordinates based on integration of
				// d = 25 / sin(M_PI_2 * (Radius * 1.2 - float(focaldist)) / Radius);
				d = -30.0 * log(fabs(2500.0
					/ sin(M_PI_2 * (Radius * 1.01 - focaldist) / Radius)
					- 2500.0 * cos(M_PI_2 * (Radius * 1.01 - focaldist) / Radius)
					/ sin(M_PI_2 * (Radius * 1.01 - focaldist) / Radius)));
				d += 30.0 * log(fabs(2500.0
					/ sin(M_PI_2 * (Radius * 1.01 - 0.0) / Radius)
					- 2500.0 * cos(M_PI_2 * (Radius * 1.01 - 0.0) / Radius)
					/ sin(M_PI_2 * (Radius * 1.01 - 0.0) / Radius)));

				// calculate texturemap coordinates using d
				x2 = d * cos(theta);
				y2 = d * sin(theta);

				switch (Mode)
				{
				case 1:
					ColorIndex = Int2Color((int)(y2 * 2));
					if ((int)x2 % 2 == 0) ColorIndex = Int2Color((int)(x2 * 2));
					break;
				case 2:
					ColorIndex = Int2Color((int)(64.0 * (sin(x2 / 8.0)
						+ sin(y2 / 8.0))));
					break;
				case 3:
					if (i % 2)
					{
						if (j % 2) ColorIndex = Int2Color((int)(7.0 * sqrt(
							(x2 - Off) * (x2 - Off) + (y2 - Off) * (y2 - Off))));
						else ColorIndex = Int2Color((int)(7.0 * sqrt((x2 - Off)
							* (x2 - Off) + (y2 + Off) * (y2 + Off))));
					}
					else
					{
						if (j % 2) ColorIndex = Int2Color((int)(7.0 * sqrt(
							(x2 + Off) * (x2 + Off) + (y2 - Off) * (y2 - Off))));
						else ColorIndex = Int2Color((int)(7.0 * sqrt((x2 + Off)
							* (x2 + Off) + (y2 + Off) * (y2 + Off))));
					}
					break;
				case 4:
					ColorIndex = Int2Color((int)(atan((double)y2
						/ (double)x2) * 255.0 / M_PI_2
						+ sqrt((double)(x2 * x2 + y2 * y2)) * 6.0));
					break;
				}

				PlotPixel(i, j, ColorIndex);
			}
		}
	}
}
