/*
palettes.cc
Copyright 2007 Matt Heinzen

This file is part of Color Machine 0.8

Color Machine is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Color Machine is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Color Machine; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "palettes.h"
#include "colormachinesdl.h"

#include "utils.h"

void SetPaletteColor(int Index, float Red, float Green, float Blue)
{
	// set the color for the specified palette index

	g_Palette[Index].r = (Uint8)(Red * 255.0f);
	g_Palette[Index].g = (Uint8)(Green * 255.0f);
	g_Palette[Index].b = (Uint8)(Blue * 255.0f);
}

void RotatePalette(bool const bForward)
{
	// rotate the palette by one position in the specified direction

	SDL_Color EndColor;
	int i;

	if (bForward)
	{
		EndColor = g_Palette[255];
		for (i = 255; i > 1; i--) g_Palette[i] = g_Palette[i - 1];
		g_Palette[1] = EndColor;
	}
	else
	{
		EndColor = g_Palette[1];
		for (i = 1; i < 255; i++) g_Palette[i] = g_Palette[i + 1];
		g_Palette[255] = EndColor;
	}

	//SDL_SetPalette(g_pScreen, SDL_PHYSPAL, g_Palette, 0, 256);
}

void RotateRed(bool const bForward)
{
	// rotate the palette's red components by one position in the specified
	// direction

	SDL_Color EndColor;
	int i;

	if (bForward)
	{
		EndColor.r = g_Palette[255].r;
		for (i = 255; i > 1; i--) g_Palette[i].r = g_Palette[i - 1].r;
		g_Palette[1].r = EndColor.r;
	}
	else
	{
		EndColor.r = g_Palette[1].r;
		for (i = 1; i < 255; i++) g_Palette[i].r = g_Palette[i + 1].r;
		g_Palette[255].r = EndColor.r;
	}

	//SDL_SetPalette(g_pScreen, SDL_PHYSPAL, g_Palette, 0, 256);
}

void RotateGreen(bool const bForward)
{
	// rotate the palette's green components by one position in the specified
	// direction

	SDL_Color EndColor;
	int i;

	if (bForward)
	{
		EndColor.g = g_Palette[255].g;
		for (i = 255; i > 1; i--) g_Palette[i].g = g_Palette[i - 1].g;
		g_Palette[1].g = EndColor.g;
	}
	else
	{
		EndColor.g = g_Palette[1].g;
		for (i = 1; i < 255; i++) g_Palette[i].g = g_Palette[i + 1].g;
		g_Palette[255].g = EndColor.g;
	}

	//SDL_SetPalette(g_pScreen, SDL_PHYSPAL, g_Palette, 0, 256);
}

void RotateBlue(bool const bForward)
{
	// rotate the palette's blue components by one position in the specified
	// direction

	SDL_Color EndColor;
	int i;

	if (bForward)
	{
		EndColor.b = g_Palette[255].b;
		for (i = 255; i > 1; i--) g_Palette[i].b = g_Palette[i - 1].b;
		g_Palette[1].b = EndColor.b;
	}
	else
	{
		EndColor.b = g_Palette[1].b;
		for (i = 1; i < 255; i++) g_Palette[i].b = g_Palette[i + 1].b;
		g_Palette[255].b = EndColor.b;
	}

	//SDL_SetPalette(g_pScreen, SDL_PHYSPAL, g_Palette, 0, 256);
}

void FadePalette_x256(int const nSteps)
{
	int i;

	for (i = 0; i < 256; i++)
	{
		if (g_Palette[i].r > nSteps) g_Palette[i].r -= nSteps;
		else g_Palette[i].r = 0;
		if (g_Palette[i].g > nSteps) g_Palette[i].g -= nSteps;
		else g_Palette[i].g = 0;
		if (g_Palette[i].b > nSteps) g_Palette[i].b -= nSteps;
		else g_Palette[i].b = 0;
	}

	SDL_SetPalette(g_pScreen, SDL_PHYSPAL, g_Palette, 0, 256);
}

void InitPaletteBlack()
{
	int i;

	for (i = 0; i < 256; i++) SetPaletteColor(i, 0.0f, 0.0f, 0.0f);
}

void InitPaletteRGBW()
{
	// map palette values for "Red-Green-Blue-White" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, i / 63.0f, 0.0f, 0.0f);
		SetPaletteColor(i + 64, 0.0f, i / 63.0f, 0.0f);
		SetPaletteColor(i + 128, 0.0f, 0.0f, i / 63.0f);
		SetPaletteColor(i + 192, i / 63.0f, i / 63.0f, i / 63.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteRedPulse()
{
	// map palette values for "Red Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, i / 63.0f, 0.0f, 0.0f);
		SetPaletteColor(i + 64, 1.0f, i / 63.0f, i / 63.0f);
		SetPaletteColor(i + 128, 1.0f, 1.0f - i / 63.0f, 1.0f - i / 63.0f);
		SetPaletteColor(i + 192, 1.0f - i / 63.0f, 0.0f, 0.0f);
	}

	// TEST: sine waves
	/**
	for (i = 0; i < 85; i++) SetPaletteColor(i, sin(i * M_PI / 256.0), 0.0f, 0.0f);
	for (i = 85; i < 171; i++) SetPaletteColor(i, sin(i * M_PI / 256.0),
		sin((i - 85) * M_PI / 86.0), sin((i - 85) * M_PI / 86.0));
	for (i = 171; i < 256; i++) SetPaletteColor(i, sin(i * M_PI / 256.0), 0.0f, 0.0f);
	/**
	for (i = 0; i < 64; i++) SetPaletteColor(i, sin(i * M_PI / 128.0), 0.0f, 0.0f);
	for (i = 64; i < 192; i++) SetPaletteColor(i, 1.0f,
		sin((i - 64) * M_PI / 128.0), sin((i - 64) * M_PI / 128.0));
	for (i = 192; i < 256; i++) SetPaletteColor(i, sin(-i * M_PI / 128.0), 0.0f, 0.0f);
	/**/

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteGreenPulse()
{
	// map palette values for "Green Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, 0.0f, i / 63.0f, 0.0f);
		SetPaletteColor(i + 64, i / 63.0f, 1.0f, i / 63.0f);
		SetPaletteColor(i + 128, 1.0f - i / 63.0f, 1.0f, 1.0f - i / 63.0f);
		SetPaletteColor(i + 192, 0.0f, 1.0f - i / 63.0f, 0.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteBluePulse()
{
	// map palette values for "Blue Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, 0.0f, 0.0f, (float)i / 63.0f);
		SetPaletteColor(i + 64, (float)i / 63.0f, (float)i / 63.0f, 1.0f);
		SetPaletteColor(i + 128, 1.0f - (float)i / 63.0f,
			1.0f - (float)i / 63.0f, 1.0f);
		SetPaletteColor(i + 192, 0.0f, 0.0f, 1.0f - (float)i / 63.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteYellowPulse()
{
	// map palette values for "Yellow Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, i / 63.0f, i / 63.0f, 0.0f);
		SetPaletteColor(i + 64, 1.0f, 1.0f, i / 63.0f);
		SetPaletteColor(i + 128, 1.0f, 1.0f, 1.0f - i / 63.0f);
		SetPaletteColor(i + 192, 1.0f - i / 63.0f, 1.0f - i / 63.0f, 0.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteCyanPulse()
{
	// map palette values for "Cyan Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, 0.0f, i / 63.0f, i / 63.0f);
		SetPaletteColor(i + 64, i / 63.0f, 1.0f, 1.0f);
		SetPaletteColor(i + 128, 1.0f - i / 63.0f, 1.0f, 1.0f);
		SetPaletteColor(i + 192, 0.0f, 1.0f - i / 63.0f, 1.0f - i / 63.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteMagentaPulse()
{
	// map palette values for "Magenta Pulse" color set

	int i;

	for (i = 0; i < 64; i++)
	{
		SetPaletteColor(i, i / 63.0f, 0.0f, i / 63.0f);
		SetPaletteColor(i + 64, 1.0f, i / 63.0f, 1.0f);
		SetPaletteColor(i + 128, 1.0f, 1.0f - i / 63.0f, 1.0f);
		SetPaletteColor(i + 192, 1.0f - i / 63.0f, 0.0f, 1.0f - i / 63.0f);
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

void InitPaletteRainbow()
{
	// map palette values for "Rainbow" color set

	int i;

	// 1-42: red-yellow
	for (i = 1; i <= 42; i++)
		SetPaletteColor(i, 1.0f, (float)i / 42.0f, 0.0f);

	// 43-85: yellow-green
	for (i = 43; i <= 85; i++)
		SetPaletteColor(i, (float)(85 - i) / 43.0f, 1.0f, 0.0f);

	// 86-128: green-cyan
	for (i = 86; i <= 128; i++)
		SetPaletteColor(i, 0.0f, 1.0f, (float)(i - 85) / 43.0f);

	// 129-171: cyan-blue
	for (i = 129; i <= 171; i++)
		SetPaletteColor(i, 0.0f, (float)(171 - i) / 43.0f, 1.0f);

	// 172-213: blue-magenta
	for (i = 172; i <= 213; i++)
		SetPaletteColor(i, (float)(i - 171) / 42.0f, 0.0f, 1.0f);

	// 214-255: magenta-red
	for (i = 214; i <= 255; i++)
		SetPaletteColor(i, 1.0f, 0.0f, (255 - i) / 42.0f);

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

// NOTE: large range to update ratio makes smoother palettes
// best values: 0.05 & 0.025 (smooth), 0.05 & 0.005 (sedate), 0.07 & 0.007 (moderate),
// 0.1 & 0.005 (moderately wild), 0.05 & 0.01 (wild), 0.1 & 0.01 (very wild)
#define COLOR_DELTA_RANGE 0.05
#define COLOR_DELTA_UPDATE 0.0025

void InitPaletteBrownian()
{
	// map palette values for "Brownian" color set

	double R, G, B, dR, dG, dB;
	int i;

	R = Random(1000) / 1000.0 * 1.0;
	G = Random(1000) / 1000.0 * 1.0;
	B = Random(1000) / 1000.0 * 1.0;
	dR = Random(100) / 100.0 * COLOR_DELTA_RANGE - COLOR_DELTA_RANGE / 2.0;
	dG = Random(100) / 100.0 * COLOR_DELTA_RANGE - COLOR_DELTA_RANGE / 2.0;
	dB = Random(100) / 100.0 * COLOR_DELTA_RANGE - COLOR_DELTA_RANGE / 2.0;

	for (i = 0; i < 128; i++) {
		SetPaletteColor(i, R, G, B);
		SetPaletteColor(255 - i, R, G, B);

		dR += Random(100) / 100.0 * COLOR_DELTA_UPDATE - COLOR_DELTA_UPDATE / 2.0;
		dG += Random(100) / 100.0 * COLOR_DELTA_UPDATE - COLOR_DELTA_UPDATE / 2.0;
		dB += Random(100) / 100.0 * COLOR_DELTA_UPDATE - COLOR_DELTA_UPDATE / 2.0;

		if (dR < -COLOR_DELTA_RANGE / 2.0) dR += COLOR_DELTA_UPDATE / 2.0;
		if (dR > COLOR_DELTA_RANGE / 2.0) dR -= COLOR_DELTA_UPDATE / 2.0;
		if (dG < -COLOR_DELTA_RANGE / 2.0) dG += COLOR_DELTA_UPDATE / 2.0;
		if (dG > COLOR_DELTA_RANGE / 2.0) dG -= COLOR_DELTA_UPDATE / 2.0;
		if (dB < -COLOR_DELTA_RANGE / 2.0) dB += COLOR_DELTA_UPDATE / 2.0;
		if (dB > COLOR_DELTA_RANGE / 2.0) dB -= COLOR_DELTA_UPDATE / 2.0;

		R += dR;
		G += dG;
		B += dB;

		if (R < 0.0)
		{
			R = 0.0;
			dR = -dR;
		}
		else if (R > 1.0)
		{
			R = 1.0;
			dR = -dR;
		}

		if (G < 0.0)
		{
			G = 0.0;
			dG = -dG;
		}
		else if (G > 1.0)
		{
			G = 1.0;
			dG = -dG;
		}

		if (B < 0.0)
		{
			B = 0.0;
			dB = -dB;
		}
		else if (B > 1.0)
		{
			B = 1.0;
			dB = -dB;
		}
	}

	SetPaletteColor(0, 0.0f, 0.0f, 0.0f);
}

/*void InitPalette__()
{
  // map palette values for "" color set

}
*/

