/*
utils.cc
Copyright 2007 Matt Heinzen

This file is part of Color Machine 0.8

Color Machine is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

Color Machine is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Color Machine; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "utils.h"
#include "colormachinesdl.h"

#include <math.h>


int Random(const int Range)
{
	//return (int)(((double)random() / (double)RAND_MAX) * (double)Range);
	//return random() % Range;

	// from http://www.thinkage.ca/english/gcos/expl/c/lib/rand.html
	// uses random() instead of rand()

	int val;

	do val = random() / (RAND_MAX / Range); while (val >= Range);
	return val;
}

Uint8 Int2Color(int A)
{
	if (A < 0) A += (-A / 255 + 1) * 255;
	return A % 255 + 1;
}

void PlotPixel(int const X, int const Y, Uint8 const ColorIndex)
{
	if (X >= 0 && X < g_DisplayWidth && Y >= 0 && Y < g_DisplayHeight)
		g_pScreenBuff[Y * g_DisplayWidth + X] = ColorIndex;
}

void DrawHLine(int X1, int X2, int Y, Uint8 const ColorIndex)
{
	int temp;

	if (Y < 0 || Y >= g_DisplayHeight) return;

	if (X1 > X2)
	{
		temp = X1;
		X1 = X2;
		X2 = temp;
	}

	if (X1 >= g_DisplayWidth || X2 < 0) return;

	if (X1 < 0) X1 = 0;
	if (X2 >= g_DisplayWidth) X2 = g_DisplayWidth - 1;

	memset(g_pScreenBuff + Y * g_DisplayWidth + X1, ColorIndex, X2 - X1 + 1);
}

void DrawVLine(int X, int Y1, int Y2, Uint8 const ColorIndex)
{
	int temp;
	int i;

	if (X < 0 || X >= g_DisplayWidth) return;

	if (Y1 > Y2) {
		temp = Y1;
		Y1 = Y2;
		Y2 = temp;
	}

	if (Y1 >= g_DisplayHeight || Y2 < 0) return;

	if (Y1 < 0) Y1 = 0;
	if (Y2 >= g_DisplayHeight) Y2 = g_DisplayHeight - 1;

	for (i = Y1; i <= Y2; i++)
		g_pScreenBuff[i * g_DisplayWidth + X] = ColorIndex;
}

void DrawLine(int X1, int Y1, int X2, int Y2, Uint8 const ColorIndex)
{
	// TODO: use a decent algorithm

	double Slope, X, Y;
	int px, py, temp;

	if (X1 == X2) DrawVLine(X1, Y1, Y2, ColorIndex);
	if (Y1 == Y2) DrawHLine(X1, X2, Y1, ColorIndex);

	if (abs(X2 - X1) >= abs(Y2 - Y1))
	{
		if (X1 > X2)
		{
			temp = X1;
			X1 = X2;
			X2 = temp;
			temp = Y1;
			Y1 = Y2;
			Y2 = temp;
		}

		Slope = (double)(Y2 - Y1) / (double)(X2 - X1);
		Y = Y1;

		for (px = X1; px <= X2; px++, Y += Slope)
			PlotPixel(px, (int)Y, ColorIndex);
	}
	else
	{
		if (Y1 > Y2)
		{
			temp = X1;
			X1 = X2;
			X2 = temp;
			temp = Y1;
			Y1 = Y2;
			Y2 = temp;
		}

		Slope = (double)(X2 - X1) / (double)(Y2 - Y1);
		X = X1;

		for (py = Y1; py <= Y2; py++, X += Slope)
			PlotPixel((int)X, py, ColorIndex);
	}
}

void DrawBox(int X1, int Y1, int X2, int Y2, Uint8 const ColorIndex)
{
	int temp, i;

	if (X1 > X2)
	{
		temp = X1;
		X1 = X2;
		X2 = temp;
	}

	if (Y1 > Y2)
	{
		temp = Y1;
		Y1 = Y2;
		Y2 = temp;
	}

	if (X1 >= g_DisplayWidth || X2 < 0 || Y1 >= g_DisplayHeight || Y2 < 0)
		return;

	if (X1 < 0) X1 = 0;
	if (Y1 < 0) Y1 = 0;
	if (X2 >= g_DisplayWidth) X2 = g_DisplayWidth - 1;
	if (Y2 >= g_DisplayHeight) Y2 = g_DisplayHeight - 1;

	for (i = Y1; i <= Y2; i++)
		memset(g_pScreenBuff + i * g_DisplayWidth + X1, ColorIndex, X2 - X1 + 1);
}

void DrawDisc(int X, int Y, int R, Uint8 ColorIndex)
{
	int MinY, MaxY, DY;
	double HalfWidth;
	int i;

	if (R <= 0) return;

	MinY = Y - R < 0 ? 0 : Y - R;
	MaxY = Y + R >= g_DisplayHeight ? g_DisplayHeight - 1 : Y + R;

	for (i = MinY; i <= MaxY; i++)
	{
		DY = Y - i;
		HalfWidth = sqrt(R * R - DY * DY);
		DrawHLine((int)(X - HalfWidth), (int)(X + HalfWidth), i, ColorIndex);
	}
}
